//****************************************************************************
//                         File module_mail_archive.c
//
//          Sendmail Milter Agent Interface Module (internal filter)
//
//    Copyright (c) Dmitry V. Stefankov, 2005. All rights reserved.
//
//****************************************************************************
//
//   $Source: /root/projects/rockmail/milter-agent/RCS/module_mail_archive.c,v $
//  $RCSfile: module_mail_archive.c,v $
//   $Author: dstef $
//     $Date: 2005-04-21 16:40:21+04 $
// $Revision: 1.1 $
//   $Locker: root $
//
//      $Log: module_mail_archive.c,v $
//      Revision 1.1  2005-04-21 16:40:21+04  dstef
//      Initial revision
//
//****************************************************************************


#include  <sys/types.h>
#include  <sys/socket.h>
#include  <unistd.h>
#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <errno.h>
#include  <time.h>
#include  <sys/stat.h>
#include  <sys/uio.h>
#include  <fcntl.h>


#include  "module.h"

/*
   Test program: 
   1) set __TEST_DRIVER__ to 1
   2) cc -o module-name module-name
   3) ./module-name args
*/
#define   __TEST_DRIVER__   0

/*------------------------------ Global data -------------------------------*/



/*-------------------------------- Local data ------------------------------*/
const char  rcsid[] = "$Id: module_mail_archive.c,v 1.1 2005-04-21 16:40:21+04 dstef Exp root $";


/*----------------------------- Functions prototypes -----------------------*/


#if __TEST_DRIVER__
int main( int argc, char ** argv )
{
    char * parameters;
    char temp_buf[512];
#else
int  process_message_by_module( char * parameters )
{
#endif
    int  result = -1;
    int  len;
    int  res;
    time_t cur_time;
    struct tm * p_tm;
    mode_t dir_mode;
    mode_t create_mode;
    int  rd = -1;
    int  wr = -1;
    size_t bread = 0;
    size_t bwrite = 0;
    ssize_t rresult;
    ssize_t wresult;
    char * p;
    char time_buf[32];
    char base_file_name[128];
    char msg_file_name[256];
    char new_file_name[512];    
    char dir_name[512];
    char new_dir_name[512];
    char cur_wrk_dir[512];
    char buf[8192];
    
#if __TEST_DRIVER__  
    if (argc < 3 ) {
        fprintf( stderr, "Usage: %s msg-file archive-dir\n", argv[0] );
        return(1);
    }
    strncpy( temp_buf, argv[1], sizeof(temp_buf) );
    strncat( temp_buf, " ", sizeof(temp_buf) );
    strncat( temp_buf, argv[2], sizeof(temp_buf) );
    parameters = temp_buf;
    printf( "parameters='%s'\n", parameters );
#else  
    if ((parameters == NULL) || (*parameters == '\0')) return (result);
#endif

    /* Get current date/time */    
    cur_time = time(NULL);
    p_tm = localtime(&cur_time);
    snprintf( time_buf, sizeof(time_buf), "%04d%02d%02d", 
              p_tm->tm_year + 1900, p_tm->tm_mon + 1, p_tm->tm_mday );
#if __TEST_DRIVER__  	      
    printf( "time_buf='%s'\n", time_buf );
#endif
    
    /* Extract message file and archive catalog */
    p = strchr( parameters, ' ' );
    len = p - parameters;
    memset( msg_file_name, 0, sizeof(msg_file_name) );
    strncpy( msg_file_name, parameters, (size_t)len );
#if __TEST_DRIVER__  	      
    printf( "msg_file_name='%s', len=%d\n", msg_file_name, len );
#endif
    while( *p == ' ') p++;
    memset( dir_name, 0, sizeof(dir_name) );    
    strncpy( dir_name, p, sizeof(dir_name) );
#if __TEST_DRIVER__  	      
    printf( "dir_name='%s'\n", dir_name );
#endif
    
    /* Make new catalog if neccessary */
    memset( new_dir_name, 0, sizeof(new_dir_name) );
    strncpy( new_dir_name, dir_name, sizeof(new_dir_name) );
    strncat( new_dir_name, "/", sizeof(new_dir_name) );
    strncat( new_dir_name, time_buf, sizeof(new_dir_name) );    
#if __TEST_DRIVER__  	      
    printf( "new_dir_name='%s'\n", new_dir_name );
#endif
    memset( cur_wrk_dir, 0, sizeof(cur_wrk_dir) );
    p = getcwd( cur_wrk_dir, sizeof(cur_wrk_dir) );
#if __TEST_DRIVER__  	      
    printf( "cur_wrk_dir='%s'\n", cur_wrk_dir );
#endif
    res = chdir( new_dir_name );
    if (res == -1) {
        dir_mode = S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH;
        res = mkdir( new_dir_name, dir_mode );
#if __TEST_DRIVER__    
        printf( "make_new_dir=%d\n" );        
#endif
        if (res == -1) {
            result = -2;
            goto done;
        }
    }
    chdir( cur_wrk_dir );
    
    /* Find filename and make new filename */
    p = strrchr( msg_file_name, '/' ); 
    if (p == NULL) p = msg_file_name;
    else p++; 
    memset( base_file_name, 0, sizeof(base_file_name) );
    strncpy( base_file_name, p, sizeof(base_file_name) );
#if __TEST_DRIVER__  	      
    printf( "base_file_name='%s'\n", base_file_name );
#endif
    memset( new_file_name, 0, sizeof(new_file_name) );
    strncpy( new_file_name, new_dir_name, sizeof(new_file_name) );
    strncat( new_file_name, "/", sizeof(new_file_name) );    
    strncat( new_file_name, base_file_name, sizeof(new_file_name) );
#if __TEST_DRIVER__  	      
    printf( "new_file_name='%s'\n", new_file_name );
#endif
    
    rd = open(msg_file_name,O_RDONLY);
#if __TEST_DRIVER__  	      
    printf( "read_open=%d\n", rd );
#endif
    if (rd == -1) {
        result = -3;
        goto done;
    }

    create_mode = S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH;
    wr = open(new_file_name,O_WRONLY|O_CREAT|O_TRUNC,create_mode);
#if __TEST_DRIVER__  	      
    printf( "write_open=%d\n", wr );
#endif
    if (wr == -1) {
        result = -4;
        goto done;
    }

    while( 1 ) {
        rresult = read( rd, buf, sizeof(buf) );    
        if (rresult <= 0) break;
	bread += rresult;
#if __TEST_DRIVER__  	      
        printf( "read_result=%lu\n", rresult );
#endif
        wresult = write( wr, buf, (size_t)rresult );    
        if (wresult <= 0) break;
	bwrite += wresult;
#if __TEST_DRIVER__  	      
        printf( "write_result=%lu\n", wresult );
#endif
    }
    if (bread == bwrite) result = 0;
    else result = -5;

done:    
    if (rd != -1) close(rd);
    if (wr != -1) close(wr);
    
#if __TEST_DRIVER__  	      
    printf( "final_result=%d\n", result );
#endif
    return(result);
}
