//****************************************************************************
//                         File module_stat_archive.c
//
//          Sendmail Milter Agent Interface Module (internal filter)
//
//    Copyright (c) Dmitry V. Stefankov, 2005. All rights reserved.
//
//****************************************************************************
//
//   $Source: /root/projects/tpmail/milter-agent/RCS/module_stat_archive.c,v $
//  $RCSfile: module_stat_archive.c,v $
//   $Author: dstef $
//     $Date: 2005-10-11 19:09:36+04 $
// $Revision: 1.2 $
//   $Locker: root $
//
//      $Log: module_stat_archive.c,v $
//      Revision 1.2  2005-10-11 19:09:36+04  dstef
//      Fixed a bug with name processing for env_from and env_to
//      (extra '')
//
//      Revision 1.1  2005-08-26 19:48:47+04  dstef
//      Initial revision
//
//****************************************************************************


#include  <sys/types.h>
#include  <sys/socket.h>
#include  <unistd.h>
#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <errno.h>
#include  <time.h>
#include  <sys/stat.h>
#include  <sys/uio.h>
#include  <fcntl.h>
#include  <sys/stat.h>


#include  "module.h"

/*
   Test program: 
   1) set __TEST_DRIVER__ to 1
   2) cc -o module-name module-name
   3) ./module-name args
*/
#define   __TEST_DRIVER__   0

/*------------------------------ Global data -------------------------------*/

const char LMTA_ACCEPT[] = "X-Spam-LMTA: Accepted";
const char LMTA_REJECT[] = "X-Spam-LMTA: Rejected";
const char RHTA_ACCEPT[] = "X-Spam-RHTA: Accepted";
const char RHTA_REJECT[] = "X-Spam-RHTA: Rejected";
const char RHTA_ACCEPT_BUT_REJECT[] = "X-Spam-RHTA: Accepted (really must be rejected)";
const char FINAL_ACCEPT[] = "X-Spam-Final: Accepted";
const char FINAL_REJECT[] = "X-Spam-Final: Rejected";
const char FINAL_ACCEPT_ACCOUNT[] = "X-Spam-Final: Accepted (passed by account)";
const char FINAL_REJECT_ACCOUNT[] = "X-Spam-Final: Rejected (passed by account)";


/*-------------------------------- Local data ------------------------------*/
const char  rcsid[] = "$Id: module_stat_archive.c,v 1.2 2005-10-11 19:09:36+04 dstef Exp root $";


/*----------------------------- Functions prototypes -----------------------*/


#if __TEST_DRIVER__
int main( int argc, char ** argv )
{
    char * parameters;
    char temp_buf[512];
#else
int  process_message_by_module( char * parameters )
{
#endif
    int  result = -1;
    int  len;
    int  clen;
    int  res;
    int  mark_count = 0;
    int  lmta_res = -1;
    int  rhta_res = -1;
    int  final_res = -1;
    int  wl_res = -1;
    int  acc_res = -1;
    u_long msg_size;
    time_t cur_time;
    struct tm * p_tm;
    struct stat sb;
    mode_t dir_mode;
    FILE * fp;
    char * s;
    char * p;
    char * last_p;
    char host_ip[32];
    char host_name[128]; 
    char env_from[256];
    char env_to[256];    
    char time_buf_stamp[32];
    char time_buf[32];
    char time_buf_mon[32];    
    char msg_file_name[256];
    char index_file_name[128];
    char dir_name[512];
    char new_dir_name[512];
    char cur_wrk_dir[512];
    char out_filename[512];
    char buf[8192];
    
#if __TEST_DRIVER__  
    if (argc < 7 ) {
        fprintf( stderr, "Usage: %s archive-dir index-file msg-file host-ip host-name 'env-from' 'env-to'\n", argv[0] );
        return(1);
    }
    strncpy( temp_buf, argv[1], sizeof(temp_buf) );
    strncat( temp_buf, " ", sizeof(temp_buf) );
    strncat( temp_buf, argv[2], sizeof(temp_buf) );
    strncat( temp_buf, " ", sizeof(temp_buf) );
    strncat( temp_buf, argv[3], sizeof(temp_buf) );
    strncat( temp_buf, " ", sizeof(temp_buf) );
    strncat( temp_buf, argv[4], sizeof(temp_buf) );
    strncat( temp_buf, " ", sizeof(temp_buf) );
    strncat( temp_buf, argv[5], sizeof(temp_buf) );
    strncat( temp_buf, " ", sizeof(temp_buf) );
    strncat( temp_buf, argv[6], sizeof(temp_buf) );
    strncat( temp_buf, " ", sizeof(temp_buf) );
    strncat( temp_buf, argv[7], sizeof(temp_buf) );
    parameters = temp_buf;
    printf( "parameters='%s'\n", parameters );
#else  
    if ((parameters == NULL) || (*parameters == '\0')) return (result);
#endif


    /* Get current date/time */    
    cur_time = time(NULL);
    p_tm = localtime(&cur_time);
    snprintf( time_buf, sizeof(time_buf), "%04d%02d%02d", 
              p_tm->tm_year + 1900, p_tm->tm_mon + 1, p_tm->tm_mday );
    snprintf( time_buf_mon, sizeof(time_buf_mon), "%04d%02d", 
              p_tm->tm_year + 1900, p_tm->tm_mon + 1 );
    snprintf( time_buf_stamp, sizeof(time_buf_stamp), "%02d:%02d:%02d", 
              p_tm->tm_hour, p_tm->tm_min, p_tm->tm_sec );
#if __TEST_DRIVER__  	      
    printf( "time_buf_day='%s'\n", time_buf );
    printf( "time_buf_mon='%s'\n", time_buf_mon );    
    printf( "time_buf_stamp='%s'\n", time_buf_stamp );        
#endif

    
    /* Extract all parameters */
    last_p = parameters;
    p = strchr( parameters, ' ' );
    if (p == NULL) {
      result = -2;
      goto done;
    }
    len = p - parameters;
    memset( dir_name, 0, sizeof(dir_name) );    
    strncpy( dir_name, parameters, (size_t)len );    
#if __TEST_DRIVER__  	      
    printf( "dir_name='%s' len=%d\n", dir_name, len );
#endif

    while( *p == ' ') p++;
    last_p = p;
    p = strchr( p, ' ' );
    if (p == NULL) {
      result = -3;
      goto done;
    }
    len = p - last_p;
    memset( index_file_name, 0, sizeof(index_file_name) );    
    strncpy( index_file_name, last_p, (size_t)len );    
#if __TEST_DRIVER__  	      
    printf( "index_file_name='%s', len=%d\n", index_file_name, len );
#endif

    while( *p == ' ') p++;
    last_p = p;
    p = strchr( p, ' ' );
    if (p == NULL) {
      result = -4;
      goto done;
    }
    len = p - last_p;
    memset( msg_file_name, 0, sizeof(msg_file_name) );    
    strncpy( msg_file_name, last_p, (size_t)len );    
#if __TEST_DRIVER__  	      
    printf( "msg_file_name='%s', len=%d\n", msg_file_name, len );
#endif

    while( *p == ' ') p++;
    last_p = p;
    p = strchr( p, ' ' );
    if (p == NULL) {
      result = -5;
      goto done;
    }
    len = p - last_p;
    memset( host_ip, 0, sizeof(host_ip) );    
    strncpy( host_ip, last_p, (size_t)len );    
#if __TEST_DRIVER__  	      
    printf( "host_ip='%s', len=%d\n", host_ip, len );
#endif

    while( *p == ' ') p++;
    last_p = p;
    p = strchr( p, ' ' );
    if (p == NULL) {
      result = -6;
      goto done;
    }
    len = p - last_p;
    memset( host_name, 0, sizeof(host_name) );    
    strncpy( host_name, last_p, (size_t)len );    
#if __TEST_DRIVER__  	      
    printf( "host_name='%s', len=%d\n", host_name, len );
#endif
    
    while( *p == ' ') p++;
    last_p = p;
    p = strchr( p, ' ' );
    if (p == NULL) {
      result = -7;
      goto done;
    }
    len = p - last_p;
    memset( env_from, 0, sizeof(env_from) );    
    s = last_p; s++;
    if (len >= 2) clen = len - 2;
    else clen = len;
    strncpy( env_from, s, (size_t)clen );    
#if __TEST_DRIVER__  	      
    printf( "env_from='%s', len=%d\n", env_from, len );
#endif

    while( *p == ' ') p++;
    last_p = p;
    p = strchr( p, ' ' );
    if (p == NULL) p = strchr( last_p, '\0' );
    if (p == NULL) {
      result = -8;
      goto done;
    }
    len = p - last_p;
    memset( env_to, 0, sizeof(env_to) );    
    s = last_p; s++;
    if (len >= 2) clen = len - 2;
    else clen = len;
    strncpy( env_to, s, (size_t)clen );    
#if __TEST_DRIVER__  	      
    printf( "env_to='%s', len=%d\n", env_to, len );
#endif


    /* Make new catalog if neccessary [Level-1] */
    memset( new_dir_name, 0, sizeof(new_dir_name) );
    strncpy( new_dir_name, dir_name, sizeof(new_dir_name) );
    strncat( new_dir_name, "/", sizeof(new_dir_name) );    
    strncat( new_dir_name, time_buf_mon, sizeof(new_dir_name) );        
#if __TEST_DRIVER__  	      
    printf( "new_dir_name='%s'\n", new_dir_name );
#endif
    memset( cur_wrk_dir, 0, sizeof(cur_wrk_dir) );
    p = getcwd( cur_wrk_dir, sizeof(cur_wrk_dir) );
#if __TEST_DRIVER__  	      
    printf( "cur_wrk_dir='%s'\n", cur_wrk_dir );
#endif
    res = chdir( new_dir_name );
    if (res == -1) {
        dir_mode = S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH;
        res = mkdir( new_dir_name, dir_mode );
#if __TEST_DRIVER__    
        printf( "make_new_dir=%d\n" );        
#endif
        if (res == -1) {
            result = -9;
            goto done;
        }
    }
    chdir( cur_wrk_dir );


    /* Make new catalog if neccessary [Level-2] */
    memset( new_dir_name, 0, sizeof(new_dir_name) );
    strncpy( new_dir_name, dir_name, sizeof(new_dir_name) );
    strncat( new_dir_name, "/", sizeof(new_dir_name) );    
    strncat( new_dir_name, time_buf_mon, sizeof(new_dir_name) );        
    strncat( new_dir_name, "/", sizeof(new_dir_name) );    
    strncat( new_dir_name, time_buf, sizeof(new_dir_name) );        
#if __TEST_DRIVER__  	      
    printf( "new_dir_name='%s'\n", new_dir_name );
#endif
    memset( cur_wrk_dir, 0, sizeof(cur_wrk_dir) );
    p = getcwd( cur_wrk_dir, sizeof(cur_wrk_dir) );
#if __TEST_DRIVER__  	      
    printf( "cur_wrk_dir='%s'\n", cur_wrk_dir );
#endif
    res = chdir( new_dir_name );
    if (res == -1) {
        dir_mode = S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH;
        res = mkdir( new_dir_name, dir_mode );
#if __TEST_DRIVER__    
        printf( "make_new_dir=%d\n" );        
#endif
        if (res == -1) {
            result = -10;
            goto done;
        }
    }
    chdir( cur_wrk_dir );


    /* Build output filename */
    memset( out_filename, 0, sizeof(out_filename) );
    strncpy( out_filename, new_dir_name, sizeof(out_filename) );
    strncat( out_filename, "/", sizeof(out_filename) );    
    strncat( out_filename, index_file_name, sizeof(out_filename) );        
#if __TEST_DRIVER__    
    printf( "out_file_name=%s\n", out_filename );        
#endif
    
    
    /* Get message size */
    memset( &sb, 0, sizeof(sb) );
    res = stat( msg_file_name, &sb );
    if (res != 0) {
      result = -11;
      goto done;
    }
    msg_size = sb.st_size;
    
    
    /* Parse message */
    fp = fopen( msg_file_name, "rt" );
    if (fp == NULL) {
      result = -12;
      goto done;
    }
    
    while( fgets(buf,sizeof(buf),fp) != NULL) {
        len = strlen(buf);
        if (len < 2 ) continue;
        
        if (strncasecmp(buf,LMTA_ACCEPT,sizeof(LMTA_ACCEPT)-1) == 0) {
           lmta_res = 1;
           mark_count++;
        }

        if (strncasecmp(buf,LMTA_REJECT,sizeof(LMTA_REJECT)-1) == 0) {
           lmta_res = 0;
           mark_count++;
        }

        if (strncasecmp(buf,RHTA_ACCEPT,sizeof(RHTA_ACCEPT)-1) == 0) {
           rhta_res = 1;
           mark_count++;
        }

        if (strncasecmp(buf,RHTA_REJECT,sizeof(RHTA_REJECT)-1) == 0) {
           rhta_res = 0;
           mark_count++;
        }

        if (strncasecmp(buf,FINAL_ACCEPT,sizeof(FINAL_ACCEPT)-1) == 0) {
           final_res = 1;
           mark_count++;
        }

        if (strncasecmp(buf,FINAL_REJECT,sizeof(FINAL_REJECT)-1) == 0) {
           final_res = 0;
           mark_count++;
        }

        if (strncasecmp(buf,FINAL_ACCEPT_ACCOUNT,sizeof(FINAL_ACCEPT_ACCOUNT)-1) == 0) {
           acc_res = 1;
           mark_count++;
        }

        if (strncasecmp(buf,FINAL_REJECT_ACCOUNT,sizeof(FINAL_REJECT_ACCOUNT)-1) == 0) {
           acc_res = 1;
           mark_count++;
        }

        if (strncasecmp(buf,RHTA_ACCEPT_BUT_REJECT,sizeof(RHTA_ACCEPT_BUT_REJECT)-1) == 0) {
           rhta_res = 0;
           final_res = 0;
           acc_res = 1;
           mark_count++;
        }

        
    }
    
    fclose( fp );


    if (mark_count == 0) wl_res = 1;
    
    
    /* Add results to stat_index_file */    
    fp = fopen( out_filename, "at" );
    if (fp == NULL) {
      result = -13;
      goto done;
    }
    fprintf( fp, "msg_time='%s' ", time_buf_stamp );
    fprintf( fp, "host_ip='%s' ", host_ip );
    fprintf( fp, "host_name='%s' ", host_name );    
    fprintf( fp, "env_from='%s' ", env_from );
    fprintf( fp, "env_to='%s' ", env_to );
    fprintf( fp, "msg_size='%lu' ", msg_size );    
    fprintf( fp, "wl_ok='%d' ", wl_res );
    fprintf( fp, "lmta_ok='%d' ", lmta_res );
    fprintf( fp, "rhta_ok='%d' ", rhta_res );
    fprintf( fp, "acc_ok='%d' ", acc_res );
    fprintf( fp, "final_ok='%d' ", final_res );
    fprintf( fp, "\n" );
    fclose(fp);    
    
    result = 0;
    
done:
    
#if __TEST_DRIVER__  	      
    printf( "final_result=%d\n", result );
#endif
    return(result);
}
