//****************************************************************************
//                         File module_test_rhta_mail.c
//
//          Sendmail Milter Agent Interface Module (internal filter)
//
//    Copyright (c) Dmitry V. Stefankov, 2005. All rights reserved.
//
//****************************************************************************
//
//   $Source: /root/projects/tpmail/milter-agent/RCS/module_test_rhta_mail.c,v $
//  $RCSfile: module_test_rhta_mail.c,v $
//   $Author: dstef $
//     $Date: 2006-02-14 20:48:36+03 $
// $Revision: 1.4 $
//   $Locker: root $
//
//      $Log: module_test_rhta_mail.c,v $
//      Revision 1.4  2006-02-14 20:48:36+03  dstef
//      Added Solaris x86 support
//
//      Revision 1.3  2005-10-17 16:53:19+04  dstef
//      Added support for timestamp check based on hostname
//
//      Revision 1.2  2005-04-23 18:02:38+04  dstef
//      Added missed <ctype.h>
//
//      Revision 1.1  2005-04-12 20:08:18+04  dstef
//      Initial revision
//
//****************************************************************************


#include  <sys/types.h>
#include  <sys/param.h>
#include  <sys/socket.h>
#include  <unistd.h>
#include  <stdio.h>
#include  <stdlib.h>
#include  <ctype.h>
#include  <string.h>
#include  <errno.h>
#include  <time.h>
#include  <sys/stat.h>


#include  "module.h"

/* Solaris x86 workaround */
#if !defined(MAXHOSTNAMELEN)
#define  MAXHOSTNAMELEN  MAXNAMELEN
#endif


/*------------------------------ Global data -------------------------------*/

const char  SEARCH_PATTERN[] = "X-Spam-RHTA: Rejected";


/*-------------------------------- Local data ------------------------------*/
const char  rcsid[] = "$Id: module_test_rhta_mail.c,v 1.4 2006-02-14 20:48:36+03 dstef Exp root $";


/*----------------------------- Functions prototypes -----------------------*/


#if !defined(strcasestr) && !defined(__FreeBSD__)
static char * strcasestr( const char * big, const char * little );

/*-
 * Copyright (c) 1990, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Chris Torek.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Find the first occurrence of find in s, ignore case.
 */
static char *
strcasestr(s, find)
	register const char *s, *find;
{
	register char c, sc;
	register size_t len;

	if ((c = *find++) != 0) {
		c = tolower((unsigned char)c);
		len = strlen(find);
		do {
			do {
				if ((sc = *s++) == 0)
					return (NULL);
			} while ((char)tolower((unsigned char)sc) != c);
		} while (strncasecmp(s, find, len) != 0);
		s--;
	}
	return ((char *)s);
}
#endif



int  process_message_by_module( char * parameters )
{
    int  result = -1;
    int  res;
    int  len;
    char * p;
    FILE * fp;
    char hostname[MAXHOSTNAMELEN];
    char buf[2048];
    
    if ((parameters == NULL) || (*parameters == '\0')) return (result);
    
    fp = fopen( parameters, "rt" );
    if (fp == NULL) {
        result = -2;
	return(result);
    }
    
    memset( hostname, 0, sizeof(hostname) );
    res = gethostname( hostname, sizeof(hostname) );
    
    result = 0;
    
    while( fgets(buf,sizeof(buf),fp) != NULL) {
         len = strlen(buf);
	 if (buf[0] == '\n') break;
	 if (buf[0] == '\r') break;
	 
	 p = strcasestr( buf, SEARCH_PATTERN );
	 if (p != NULL) {
	     p = strstr( buf, hostname );
	     if (p != NULL) {
	         result = 1;
	         break;
	     }
	 }
    }
    
    fclose(fp);
    
    return(result);
}
